//------------------------------ ANNUITY --------------------------------
//                by Carl Paulson      Cognitive Portals
//                 http://members.tripod.com/~SleepyCP/
//                          cpaulson@home.com
//-----------------------------------------------------------------------
//
//  Suggested HTML Input:
/*
      <APPLET code="Annuity.class" width=350 height=250>
          <param name="BGcolor"   value="magenta">
          <param name="FGcolor"   value="black">

          <CENTER><FONT SIZE=3>{If you can read this, your browser 
          dosen`t support Java.}</FONT><BR>
          <FONT FACE="Helvetica" COLOR="#FF0000" SIZE=7>Ordinary 
          Annuity Applet</FONT></CENTER><BR>
      </APPLET>
*/
//    Ordinary Annuity / Present Value program
//      Data Used:
//        The Number of payment periods in years
//        The Payment Ammount
//        The total ammount of the loan
//        The Interest rate (simple yearly interest)
//        The Balloon payment
//
//-----------------------------------------------------------------------

import java.util.*;
import java.net.*;

import java.awt.event.*;
import java.awt.*;
import java.applet.Applet;

//-----------------------------------------------------------------------

public class Annuity extends Applet {
 
    private boolean debug  = true ;
    private Color colorBG, colorFG;
    private int switchVar;

    private int    numPay  =   0;
    private double iAmount = 0.0,    iYInterest = 0.0;
    private double iPValue = 0.0,    iBalloon   = 0.0;

    private double DnumPay = 0.0;
    private double Ammount = 0.0,    YInterest  = 0.0;
    private double PValue  = 0.0,    Balloon    = 0.0;

    private double monthlyInterest = 0.0;

    private Panel setupNorth;
    private Panel setupSouth;
    private Panel setupEast;
    private Panel setupWest;
    private Panel pan;

    private TextField  t1, t2, t3, t4, t5;
    private String     s1, s2, s3, s4, s5, s0;
    private Label      L1, L2, L3, L4, L5, L0;
    private int        left, right, top, bottom;
    private Button     b1, b2;
    private int        iTemp;
    private Integer    ITemp;
    private String     STemp;
    private Double     DTemp;
    private double     dTemp;

    private CheckboxGroup CBgroup;

//-----------------------------------------------------------------------

    public synchronized void init() {

        colorBG    = getColorParam("BGcolor", "magenta");
        setBackground(colorBG);
        colorFG    = getColorParam("FGcolor", "black");
        setForeground(colorFG);

        setLayout(new BorderLayout(5,5));
        setupNorth = new Panel();
        setupEast  = new Panel();
        setupWest  = new Panel();

        setupSouth = new Panel();
        setupSouth.setFont(new Font("Helvetica",Font.PLAIN,10));
        setupSouth.setLayout(new GridLayout(1,5,10,2));

        CBgroup    = new CheckboxGroup();
        Checkbox CB1 = new Checkbox("Loan", CBgroup, false);
        Checkbox CB2 = new Checkbox("Payment", CBgroup, true);
        Checkbox CB3 = new Checkbox("Interest", CBgroup, false);
        Checkbox CB4 = new Checkbox("Term", CBgroup, false);
        Checkbox CB5 = new Checkbox("Balloon", CBgroup, false);
        setupSouth.add(CB1);  setupSouth.add(CB2);  setupSouth.add(CB3); 
        setupSouth.add(CB4);  setupSouth.add(CB5);

        pan = new Panel();
        pan.setLayout(new GridLayout(6,3,20,10)); //rows & cols    

        t1 = new TextField("99999",10);
        t2 = new TextField("0",10);
        t3 = new TextField("4.625",10);
        t4 = new TextField("20",10);
        t5 = new TextField("0",10);

        b1 = new Button("Calculate");
        b2 = new Button("  Reset  ");

        s0 = new String("Annuity Calculator");
        s1 = new String("  Loan     ");
        s2 = new String("  Payment  ");
        s3 = new String("  Interest ");
        s4 = new String("  Loan Term");
        s5 = new String("  Balloon  ");

        L0 = new Label(s0);
        L1 = new Label(s1);
        L2 = new Label(s2);
        L3 = new Label(s3);
        L4 = new Label(s4);
        L5 = new Label(s5);

//      ---------- Inputs ----------

        switchVar = 5;

//      ----------

        numPay  = 30;
        DnumPay = 30.0;
        ITemp   = new Integer(numPay); 
        STemp   = ITemp.toString();
        t4.setText(STemp);

//      ----------

        Ammount = 0.0;
        DTemp   = new Double(Ammount);   // Wrap
        STemp   = DTemp.toString();      // convert to String
        t2.setText(STemp);               // Fill Text Field
        switchVar = 3;                   // Initial field to calculate

//      ----------

        PValue  = 100000.0;
        DTemp   = new Double(PValue); 
        STemp   = DTemp.toString();
        t1.setText(STemp);

//      ----------

        YInterest = 4.875;
        DTemp     = new Double(YInterest); 
        STemp     = DTemp.toString();
        t3.setText(STemp);
        monthlyInterest = YInterest / 1200.0;  // fraction

//      ----------

        Balloon = 0.0;
        DTemp   = new Double(Balloon);  
        STemp   = DTemp.toString();
        t5.setText(STemp);

//      ----------

        if ( debug ) System.out.println("calculation parameter = "+switchVar);
        Calculate();
        makeGUI();
    }

// ----------------------------------------------------------------------

    private void Calculate() { 

        double Fac1, Fac2, Fac3, Fac4, Fac5, Fac, dFac, Delta, D;
        double one,  num;
        int itno;

        one = 1.0;
        num = 12.0*100.0;

        if ( debug ) System.out.println("switchVar = "+switchVar);
        if ( debug ) System.out.println("PValue = "+PValue);
        if ( debug ) System.out.println("Ammount = "+Ammount);
        if ( debug ) System.out.println("DnumPay = "+DnumPay+" numPay = "+numPay);
        if ( debug ) System.out.println("Interest = "+YInterest+"monthlyInterest = "+monthlyInterest);
        if ( debug ) System.out.println("Balloon = "+Balloon);

        switch (switchVar) {

            case 1:   //  calculate the number of payments
                Fac1    = PValue/Ammount - one/monthlyInterest;
                Fac2    = Balloon/Ammount - one/monthlyInterest;
                DnumPay = Math.log(Fac1/Fac2)/(-Math.log(one + monthlyInterest))/12.0;
                if (DnumPay < 1.0) {
                    DnumPay = 1.0;
                    numPay  = 1;
                    ITemp   = new Integer(numPay); 
                    STemp   = ITemp.toString();
                    t4.setText(STemp);

                    Fac1    = one + monthlyInterest;
                    Fac2    = Math.pow(Fac1,(-12.0));
                    Fac4    = (one - Fac2)/monthlyInterest;
                    Fac3    = Balloon*Fac2;
                    Ammount = (PValue - Fac3)/Fac4;
                    if ( debug ) System.out.println("Monthly Payment = "+Ammount);
                    DTemp   = new Double(Ammount);
                    STemp   = DTemp.toString();
                    t2.setText(STemp);
                } else {
                    numPay  = (int) (DnumPay + 0.5);
                    if ( debug ) System.out.println("Number of Payments = "+numPay);
                    ITemp   = new Integer(numPay); 
                    STemp   = ITemp.toString();
                    t4.setText(STemp);
                }
                break;
        
            case 2:   // calculate the Loan Ammount
                Fac1   = one + monthlyInterest;
                Fac2   = Math.pow(Fac1,(-12.0*DnumPay));
                Fac4   = (one - Fac2)/monthlyInterest;
                Fac3   = Balloon*Fac2;
                PValue = Ammount*Fac4 + Fac3;
                if ( debug ) System.out.println("Loan Ammount (Present Value) = "+PValue);
                DTemp  = new Double(PValue);
                STemp  = DTemp.toString();
                t1.setText(STemp);
                break;

            case 3:   // calculate the payment
                Fac1    = one + monthlyInterest;
                Fac2    = Math.pow(Fac1,(-12.0*DnumPay));
                Fac4    = (one - Fac2)/monthlyInterest;
                Fac3    = Balloon*Fac2;
                Ammount = (PValue - Fac3)/Fac4;
                if ( debug ) System.out.println("Monthly Payment = "+Ammount);
                DTemp   = new Double(Ammount);
                STemp   = DTemp.toString();
                t2.setText(STemp);
                break;
        
            case 4:   // calculate the yearly interest
 
                monthlyInterest = (float)(0.015);  // initial guess for iteration
                for (itno=1; itno<=25; itno++)  {
                    Fac1  = one + monthlyInterest;
                    Fac2  = Math.pow((double)Fac1,(double)(-12.0*DnumPay));
                    Fac3  = Fac2/Fac1;
                    Fac4  = 12.0*(-DnumPay*Fac3);
                    Fac5  = PValue/Ammount - Balloon*Fac2/Ammount;
                    Fac   = monthlyInterest - (one - Fac2)/Fac5;
                    dFac  = one + Fac4/Fac5 - 
                           (one - Fac2)*(Balloon*Fac4/Ammount)/(Fac5*Fac5);
                    Delta = (- Fac/dFac);
                    monthlyInterest += Delta;
                    if ( debug ) System.out.println("Int = "+monthlyInterest+" Delta = "+Delta);
                    if (Math.abs(Delta) < 0.0001) break;
                }
                if (itno > 24 ) {
                    if ( debug ) System.out.println("Calculation ERROR; Iteration Max exceeded!");
                    return;
                }

                YInterest = num*monthlyInterest;
                if ( debug ) System.out.println("Loan Ammount (Present Value) = "+PValue);
                DTemp = new Double(YInterest); 
                STemp = DTemp.toString();
                t3.setText(STemp);
                break;
        
            case 5:   // calculate the balloon payment
                Fac1    = one + monthlyInterest;
                Fac2    = Math.pow((double)Fac1,(double)(-12.0*DnumPay));
                Fac3    = (one - Fac2)/monthlyInterest;
                Balloon =(PValue - Ammount*Fac3)/Fac2;
                if ( debug ) System.out.println("Balloon Payment = "+Balloon);
                DTemp   = new Double(Balloon);
                STemp   = DTemp.toString();
                t5.setText(STemp);
                break;
        
            default:
                if ( debug ) System.out.println("Problem in the SWITCH routines!");
        }

    }

// ----------------------------------------------------------------------

    protected void makeGUI() {

        setupNorth.setFont(new Font("Helvetica",Font.BOLD,24));
        setupNorth.add(L0);
        add("North", setupNorth);

        setupEast.add(new Label(new String("  ")));
        add("East", setupEast);
        setupWest.add(new Label(new String("  ")));
        add("West", setupWest);

        add("South", setupSouth);

        pan.setFont(new Font("Helvetica",Font.BOLD,12));
// ---------- Row 1 ---------- Loan Amount
        pan.add(L1);
        pan.add(t1);
        pan.add(new Label(new String("    Dollars")));
// ---------- Row 2 ---------- Monthly Payment
        pan.add(L2);
        pan.add(t2);
        pan.add(new Label(new String("    Dollars")));
// ---------- Row 3 ---------- Simple Interest
        pan.add(L3);
        pan.add(t3);
        pan.add(new Label(new String("    Percent")));
// ---------- Row 4 ---------- Loan Term
        pan.add(L4);
        pan.add(t4);
        pan.add(new Label(new String("    -Years-")));
// ---------- Row 5 ---------- Balloon Payment
        pan.add(L5);
        pan.add(t5);
        pan.add(new Label(new String("    Dollars")));
// ---------- Row 7 ---------- Buttons
        pan.add(b1);
        pan.add(new Label(""));
        pan.add(b2);
// ----------
        add("Center",pan);

    }

// ----------------------------------------------------------------------

//========================== Program  Control ===========================

    public Annuity() { }
    public void start() { }
    public void stop() { }
//  public void run() { }

//-----------------------------------------------------------------------

    public boolean action (Event evt, Object arg) {

        if(evt.target.equals(b1)) {

// ---------          Implement calculation routine
// ---------          Check Text Field Values

            try { 
                PValue = (double) Float.valueOf(t1.getText()).floatValue();
            } catch (Exception e) {
                PValue = 0.0;
                t1.setText(new String("0"));
                System.out.println("Exception: Loan Amount reset to 0");
                return false;
            }
            DTemp = new Double(PValue); 
            STemp = DTemp.toString(); 
            t1.setText(new String(STemp));
// ---------
            try { 
                Ammount = (double) Float.valueOf(t2.getText()).floatValue();
            } catch (Exception e) {
                Ammount = 0.0;
                t2.setText(new String("0"));
                System.out.println("Exception: Payment reset to 0");
                return false;
            }
            DTemp = new Double(Ammount); 
            STemp = DTemp.toString(); 
            t2.setText(new String(STemp));
// ---------
            try { 
                YInterest = (double) Float.valueOf(t3.getText()).floatValue();
            } catch (Exception e) {
                YInterest = 0.0;
                monthlyInterest = 0.0;
                t3.setText(new String("0"));
                System.out.println("Exception: Loan Ammount reset to 0");
                return false;
            }
            monthlyInterest = YInterest / 1200.0;
            DTemp = new Double(YInterest); 
            STemp = DTemp.toString(); 
            t3.setText(new String(STemp));
// ---------
            try { 
                numPay  = Integer.parseInt(t4.getText());
            } catch (Exception e) {
                numPay  = 0;
                DnumPay = 0.0;
                t4.setText(new String("0"));
                System.out.println("Exception: Loan Term reset to 0");
                return false;
            }
            DnumPay = (double) numPay;
            DTemp   = new Double(DnumPay);
            STemp   = DTemp.toString(); 
            t4.setText(new String(STemp));
// ---------
            try { 
                Balloon = (double) Float.valueOf(t5.getText()).floatValue();
            } catch (Exception e) {
                t5.setText(new String("0"));
                System.out.println("Exception: Loan Ammount reset to 0");
                return false;
            }
            DTemp   = new Double(Balloon);
            STemp   = DTemp.toString(); 
            t5.setText(new String(STemp));

//========== - = = = = = = = = = = = = = = = = =
// ---------          Determine Variable to calculate

            Checkbox cur = CBgroup.getCurrent();
            String curSt = cur.getLabel();
// ---------
            if (curSt.equals("Payment")) {
                switchVar = 3;
                Ammount = 0.0;
                t2.setText(new String("0"));
                dTemp   = DnumPay*YInterest*PValue;
                if (dTemp < 0.1) {
                    if ( debug ) System.out.println("Insufficient data");
                    return false; }
            } else if (curSt.equals("Loan")) {
                switchVar = 2;
                PValue  = 0.0;
                t1.setText(new String("0"));
                dTemp   = DnumPay*YInterest*Ammount;
                if (dTemp < 0.1) {
                    if ( debug ) System.out.println("Insufficient data");
                    return false; }
            } else if (curSt.equals("Interest")) {
                switchVar = 4;
                YInterest = 0.0;
                t3.setText(new String("0"));
                monthlyInterest = 0.0;
                dTemp   = DnumPay*PValue*Ammount;
                if (dTemp < 0.1) {
                    if ( debug ) System.out.println("Insufficient data");
                    return false; }
            } else if (curSt.equals("Term")) {
                switchVar = 1;
                DnumPay = 0.0;
                numPay  = 0;
                t4.setText(new String("0"));
                dTemp   = YInterest*PValue*Ammount;
                if (dTemp < 0.1) {
                    if ( debug ) System.out.println("Insufficient data");
                    return false; }
            } else if (curSt.equals("Balloon")) {
                switchVar = 5;
                Balloon = 0.0;
                t5.setText(new String("0"));
                dTemp   = DnumPay*YInterest*PValue*Ammount;
                if (dTemp < 0.1) {
                    if ( debug ) System.out.println("Insufficient data");
                    return false; }
            } else {
                if ( debug ) System.out.println("No idea how I could get here.");
                return false;
            }

//========== - = = = = = = = = = = = = = = = = =

            Calculate();

// *********

        } else if (evt.target.equals(b2)) { 
            // implement reset routine
            numPay    = 0;
            DnumPay   = 0.0;
            Ammount   = 0.0;
            PValue    = 0.0;
            YInterest = 0.0;
            Balloon   = 0.0;
            monthlyInterest = 0.0;

            t1.setText(new String("0"));
            t2.setText(new String("0"));
            t3.setText(new String("0"));
            t4.setText(new String("0"));
            t5.setText(new String("0"));

//          makeGUI();

        } else return false;
        return true;
    }

// ----------------------------------------------------------------------
// ------------------------- getInteger ---------------------------------

    public int getIntParam(String i, int def_I, int min_i, int max_i) {
        int i_in;
        try {
            i_in = Integer.parseInt(getParameter(i));
            if ((i_in < min_i)||(i_in > max_i)) throw new Exception();
        } catch (Exception e) {
            System.out.println(i+" invalid; using default value: "+def_I);
            i_in = def_I;
        }	
        return i_in;
    }

// -------------------------- getColor ----------------------------------

    public Color getColorParam(String c, String def_C) {

        Color Color1 = Color.lightGray;      // default color
        if (def_C != null) {
            if      (def_C.equalsIgnoreCase("black"))     Color1 = Color.black;
            else if (def_C.equalsIgnoreCase("blue"))      Color1 = Color.blue;
            else if (def_C.equalsIgnoreCase("cyan"))      Color1 = Color.cyan;
            else if (def_C.equalsIgnoreCase("darkGray"))  Color1 = Color.darkGray;
            else if (def_C.equalsIgnoreCase("gray"))      Color1 = Color.gray;
            else if (def_C.equalsIgnoreCase("green"))     Color1 = Color.green;
            else if (def_C.equalsIgnoreCase("lightGray")) Color1 = Color.lightGray;
            else if (def_C.equalsIgnoreCase("magenta"))   Color1 = Color.magenta;
            else if (def_C.equalsIgnoreCase("orange"))    Color1 = Color.orange;
            else if (def_C.equalsIgnoreCase("pink"))      Color1 = Color.pink;
            else if (def_C.equalsIgnoreCase("red"))       Color1 = Color.red;
            else if (def_C.equalsIgnoreCase("white"))     Color1 = Color.white;
            else if (def_C.equalsIgnoreCase("yellow"))    Color1 = Color.yellow;
        }

        StringTokenizer st;
        String rgbDelim = ":,.";
        String c_str = getParameter(c);
        if (c_str != null) st = new StringTokenizer(c_str, rgbDelim);
        else return Color1;

        if      (c_str.equalsIgnoreCase("black"))     return Color.black;
        else if (c_str.equalsIgnoreCase("blue"))      return Color.blue;
        else if (c_str.equalsIgnoreCase("cyan"))      return Color.cyan;
        else if (c_str.equalsIgnoreCase("darkGray"))  return Color.darkGray;
        else if (c_str.equalsIgnoreCase("gray"))      return Color.gray;
        else if (c_str.equalsIgnoreCase("green"))     return Color.green;
        else if (c_str.equalsIgnoreCase("lightGray")) return Color.lightGray;
        else if (c_str.equalsIgnoreCase("magenta"))   return Color.magenta;
        else if (c_str.equalsIgnoreCase("orange"))    return Color.orange;
        else if (c_str.equalsIgnoreCase("pink"))      return Color.pink;
        else if (c_str.equalsIgnoreCase("red"))       return Color.red;
        else if (c_str.equalsIgnoreCase("white"))     return Color.white;
        else if (c_str.equalsIgnoreCase("yellow"))    return Color.yellow;

        else if (st.countTokens() == 3) {
            Integer r = new Integer(st.nextToken());
            Integer g = new Integer(st.nextToken());
            Integer b = new Integer(st.nextToken());
            Color1    = new Color(r.intValue(), g.intValue(), b.intValue());
            return Color1;
            }
        return Color1;
    }

// ------------------------------ getFloat ------------------------------

    public float getFloatParam(String f, float def_F, float min_f, float max_f) {
        float ff;
        try {
            String f_str = getParameter(f);
            ff = Float.valueOf(f_str).floatValue();
            if ((ff < min_f)||(ff > max_f)) throw new Exception();
        } catch (Exception e) {
                System.out.println(f+" Invalid; using default value: "+def_F);
                ff = def_F;
        }	
        return ff;
    }

// ----------------------------------------------------------------------

}

//***********************************************************************
